#include "gMap.h"

//This is a function that gets the level pointer,
//based on Sting's function.
Level* GetLevelPointer(ActMisc *pActMisc, int nLevel)
{
	for (Level *pLevel = pActMisc->pLevelFirst; pLevel; pLevel = pLevel->pNextLevel)
	{
		if (!pLevel)
			continue;

		if ((pLevel->dwLevelNo == nLevel) && (pLevel->dwSizeX > 0))
			return pLevel;
	}
	return D2COMMON_GetLevel(pActMisc, nLevel);
}

/* bool RevealLevel(Level *pLevel)
 *	Reveals all the rooms in the level.
 */
bool RevealLevel(Level *pLevel)
{
	//Initalize the automap layer to the level given
	InitAutomapLayer(pLevel->dwLevelNo);

	UnitAny *pUnit = D2CLIENT_GetPlayerUnit ();

	//Loop each room in this level
	for (Room2 *pRoom2 = pLevel->pRoom2First; pRoom2; pRoom2 = pRoom2->pRoom2Other)
	{
		//A bool to check if we have added Room Data
		bool nAdded = false;

		//Add the room data if it is needed
		if (!pRoom2->pRoom1)
		{
			D2COMMON_AddRoomData(pLevel->pMisc->pAct,pLevel->dwLevelNo, pRoom2->dwPosX, pRoom2->dwPosY, pUnit->pPath->pRoom1);
			nAdded = true;
		}

		//Insure that we have a valid room before revealing it.
		if (!pRoom2->pRoom1)
			continue;
	
		//Reveal the map!
		D2CLIENT_RevealAutomapRoom(pRoom2->pRoom1, TRUE, (*p_D2CLIENT_AutomapLayer));

		//Remove Room Data if we added it!
		if (nAdded)
			D2COMMON_RemoveRoomData(pLevel->pMisc->pAct, pLevel->dwLevelNo, pRoom2->dwPosX, pRoom2->dwPosY, D2CLIENT_GetPlayerUnit()->pPath->pRoom1);
	}

	return true;
}


/* BOOL RevealAct ()
 * Reveals every area in the act.
 */
BOOL RevealAct ()
{
	//Grab the player unit now, so we don't call this 100 times
	UnitAny *pUnit = D2CLIENT_GetPlayerUnit ();

	//The level id's for each town and the last one is the final map
	// Fixed by TheAce to reveal last levels in Act 5
	int townLevels[6] = {1, 40, 75, 103, 109, 133};

	
	//We are going to loop every level in the current act.
	for (int nLevel = townLevels[pUnit->dwAct] + 1; nLevel < townLevels[pUnit->dwAct + 1]; nLevel++)
	{
		/*Level *pLevel = GetLevelPointer(pUnit->pAct->pMisc, nLevel);

		//Insure we have a good level struct
		if(!pLevel)
			continue;*/
		Level * pLevel;

		while (!(pLevel = GetLevelPointer(pUnit->pAct->pMisc, nLevel)));


		//Init the level so we can add map to it
		if(!pLevel->pRoom2First)
			D2COMMON_InitLevel(pLevel);

		//Reveal the level!
		RevealLevel(pLevel);
	}

	//Reset the map layer to the current area
	InitAutomapLayer(D2CLIENT_GetPlayerUnit()->pPath->pRoom1->pRoom2->pLevel->dwLevelNo);

	return true;
}

//----Next functions have been taken from D2BS----

DWORD __declspec(naked) __fastcall D2CLIENT_InitAutomapLayer_STUB(DWORD nLayerNo)
{
	__asm 
	{
		push eax;
		mov eax, ecx;
		call D2CLIENT_InitAutomapLayer_I;
		pop eax;
		ret;
	}
}

AutomapLayer* InitAutomapLayer(DWORD levelno)
{
	AutomapLayer2 *pLayer = D2COMMON_GetLayer(levelno);
	return D2CLIENT_InitAutomapLayer(pLayer->nLayerNo);
}

DWORD GetPlayerArea(VOID)
{
	if(D2CLIENT_GetPlayerUnit())
		if(D2CLIENT_GetPlayerUnit()->pPath)
			if(D2CLIENT_GetPlayerUnit()->pPath->pRoom1)
				if(D2CLIENT_GetPlayerUnit()->pPath->pRoom1->pRoom2)
					if(D2CLIENT_GetPlayerUnit()->pPath->pRoom1->pRoom2->pLevel)
						return D2CLIENT_GetPlayerUnit()->pPath->pRoom1->pRoom2->pLevel->dwLevelNo;
	return NULL;
}

BOOL GameReady(VOID)
{
	if(D2CLIENT_GetPlayerUnit())
		if(D2CLIENT_GetPlayerUnit()->pInventory)
			if(D2CLIENT_GetPlayerUnit()->pPath)
				if(D2CLIENT_GetPlayerUnit()->pPath->xPos)
					if(D2CLIENT_GetPlayerUnit()->pPath->pRoom1)
						if(D2CLIENT_GetPlayerUnit()->pPath->pRoom1->pRoom2)
							if(D2CLIENT_GetPlayerUnit()->pPath->pRoom1->pRoom2->pLevel)
								if(GetPlayerArea() != NULL)
									return TRUE;
	return FALSE;
}
